@@ -1,9 +1,3 @@
-from pipelime.cli.utils import (
-    print_command_op_stage_info,
-    print_commands_ops_stages_list,
-    pl_print,
-)
-
 import typing as t
 
 
@@ -83,9 +77,10 @@ def run():
             run()
         ```
     """
-    from pipelime.utils.inspection import MyCaller
     from pathlib import Path
 
+    from pipelime.utils.inspection import MyCaller
+
     caller = MyCaller()
     caller_path = Path(caller.filename)
 
