@@ -1,9 +1,11 @@
 import typing as t
+
 import pydantic as pyd
 
 import pipelime.commands.interfaces as pl_interfaces
-from pipelime.piper import PipelimeCommand, PiperPortType
 import pipelime.utils.pydantic_types as pl_types
+from pipelime.piper import PipelimeCommand, PiperPortType
+from pipelime.stages import StageInput
 
 
 class TimeItCommand(PipelimeCommand, title="timeit"):
@@ -82,9 +84,10 @@ class TimeItCommand(PipelimeCommand, title="timeit"):
     )
 
     def run(self):
-        import time
         import shutil
-        from pipelime.sequences import build_pipe, SamplesSequence
+        import time
+
+        from pipelime.sequences import SamplesSequence, build_pipe
 
         if self.input is None and self.operations is None:
             raise ValueError("No input dataset or operation defined.")
@@ -176,7 +179,7 @@ class PipeCommand(PipelimeCommand, title="pipe"):
         return v
 
     def run(self):
-        from pipelime.sequences import build_pipe, SamplesSequence
+        from pipelime.sequences import SamplesSequence, build_pipe
 
         seq = SamplesSequence if self.input is None else self.input.create_reader()
         seq = build_pipe(self.operations.value, seq)  # type: ignore
@@ -192,8 +195,11 @@ class PipeCommand(PipelimeCommand, title="pipe"):
 
 
 class CloneCommand(PipelimeCommand, title="clone"):
-    """Clone a dataset. You can use this command to create a local copy of a dataset
-    hosted on a remote data lake by disabling the `REMOTE_FILE` serialization option."""
+    """Clone a dataset.
+
+    You can use this command to create a local copy of a dataset
+    hosted on a remote data lake by disabling the `REMOTE_FILE` serialization option.
+    """
 
     input: pl_interfaces.InputDatasetInterface = (
         pl_interfaces.InputDatasetInterface.pyd_field(
@@ -390,6 +396,7 @@ class RemoteCommandBase(PipelimeCommand):
 
 class AddRemoteCommand(RemoteCommandBase, title="remote-add"):
     """Upload samples to one or more remotes.
+
     Slicing and key options filter the samples to upload,
     but the whole dataset is always written out.
     """
@@ -408,8 +415,10 @@ class AddRemoteCommand(RemoteCommandBase, title="remote-add"):
 
 class RemoveRemoteCommand(RemoteCommandBase, title="remote-remove"):
     """Remove one or more remote from a dataset.
+
     Slicing and key options filter the samples,
     but the whole dataset is always written out.
+
     NB: data is not removed from the remote data lake.
     """
 
@@ -577,7 +586,7 @@ class ValidateCommand(PipelimeCommand, title="validate"):
 class MapCommand(PipelimeCommand, title="map"):
     """Apply a stage on a dataset."""
 
-    stage: t.Union[str, t.Mapping[str, t.Mapping[str, t.Any]]] = pyd.Field(
+    stage: StageInput = pyd.Field(
         ...,
         alias="s",
         description=(
@@ -657,8 +666,6 @@ class SortCommand(PipelimeCommand, title="sort"):
         return x.deep_get(self.sort_key)
 
     def run(self):
-        from pipelime.choixe.utils.imports import import_symbol
-
         if (self.sort_key is None) == (self.sort_fn is None):
             raise ValueError("You should define either `sort_key` or `sort_fn`")
 
@@ -712,20 +719,23 @@ class FilterCommand(PipelimeCommand, title="filter"):
         alias="g"
     )
 
+    @pyd.validator("filter_fn", always=True)
+    def _check_filters(
+        cls, v: t.Optional[pl_types.CallableDef], values: t.Mapping[str, t.Any]
+    ) -> t.Optional[pl_types.CallableDef]:
+        fquery = values.get("filter_query", None)
+        if (v is None) == (fquery is None):
+            raise ValueError("You should define either `filter_query` or `filter_fn`")
+        return v
+
     def _filter_key_fn(self, x):
         return x.match(self.filter_query)
 
     def run(self):
-        from pipelime.choixe.utils.imports import import_symbol
         from pipelime.sequences import DataStream
 
-        if (self.filter_query is None) == (self.filter_fn is None):
-            raise ValueError("You should define either `filter_query` or `filter_fn`")
-
         filter_fn = (
-            self._filter_key_fn
-            if self.filter_query is not None
-            else self.filter_fn.value  # type: ignore
+            self._filter_key_fn if self.filter_fn is None else self.filter_fn.value
         )
 
         # multi-processing friendly filtering
@@ -753,3 +763,266 @@ class FilterCommand(PipelimeCommand, title="filter"):
             sample_fn=writer_helper,
             track_message=f"Filtering data ({len(seq)} samples)",
         )
+
+
+class SliceCommand(PipelimeCommand, title="slice"):
+    """Extract a subset of samples from a dataset."""
+
+    input: pl_interfaces.InputDatasetInterface = (
+        pl_interfaces.InputDatasetInterface.pyd_field(
+            alias="i", piper_port=PiperPortType.INPUT
+        )
+    )
+
+    output: pl_interfaces.OutputDatasetInterface = (
+        pl_interfaces.OutputDatasetInterface.pyd_field(
+            alias="o", piper_port=PiperPortType.OUTPUT
+        )
+    )
+
+    slice: pl_interfaces.ExtendedInterval = pl_interfaces.ExtendedInterval.pyd_field(
+        alias="s"
+    )
+
+    shuffle: t.Union[bool, pyd.PositiveInt] = pyd.Field(
+        False,
+        alias="shf",
+        description=(
+            "Shuffle the dataset before slicing. Optionally specify the random seed."
+        ),
+    )
+
+    grabber: pl_interfaces.GrabberInterface = pl_interfaces.GrabberInterface.pyd_field(
+        alias="g"
+    )
+
+    def run(self):
+        seq = self.input.create_reader()
+        if self.shuffle:
+            seq = seq.shuffle(
+                seed=self.shuffle if not isinstance(self.shuffle, bool) else None
+            )
+        seq = seq.slice(
+            start=self.slice.start, stop=self.slice.stop, step=self.slice.step
+        )
+        seq = self.output.append_writer(seq)
+        self.grabber.grab_all(
+            seq,
+            grab_context_manager=self.output.serialization_cm(),
+            keep_order=False,
+            parent_cmd=self,
+            track_message=f"Slicing dataset ({len(seq)} samples)",
+        )
+
+
+class SetMetadataCommand(FilterCommand, title="set-meta"):
+    """Set a metadata for some or all samples in a dataset.
+
+    The metadata is set only on samples selected by a dictquery or a filter function.
+    """
+
+    filter_fn: t.Optional[pl_types.CallableDef] = pyd.Field(
+        None,
+        alias="f",
+        description=(
+            "A `class.path.to.func` (or `file.py:func`) to "
+            "a callable returning True for any valid sample.\n"
+            "Accepted signatures:\n"
+            "  `() -> bool`\n"
+            "  `(index: int) -> bool`\n"
+            "  `(index: int, sample: Sample) -> bool`\n"
+            "  `(index: int, sample: Sample, source: SamplesSequence) -> bool`."
+        ),
+    )
+
+    key_path: str = pyd.Field(
+        ..., alias="k", description="The metadata key in pydash dot notation."
+    )
+    value: pl_types.YamlInput = pyd.Field(
+        None, alias="v", description="The value to set, ie, any valid yaml/json value."
+    )
+
+    def _filter_key_fn(self, idx, x):
+        return x.match(self.filter_query)
+
+    def run(self) -> None:
+        from pipelime.stages import StageSetMetadata
+
+        seq = self.input.create_reader()
+        seq = seq.map_if(
+            stage=StageSetMetadata(key_path=self.key_path, value=self.value),
+            condition=(
+                self._filter_key_fn if self.filter_fn is None else self.filter_fn.value
+            ),
+        )
+        seq = self.output.append_writer(seq)
+        self.grabber.grab_all(
+            seq,
+            grab_context_manager=self.output.serialization_cm(),
+            keep_order=False,
+            parent_cmd=self,
+            track_message=f"Setting metadata ({len(seq)} samples)",
+        )
+
+
+class FilterDuplicatesCommand(PipelimeCommand, title="filter-duplicates"):
+    """Filter duplicated samples based on the hash of a set of items."""
+
+    input: pl_interfaces.InputDatasetInterface = (
+        pl_interfaces.InputDatasetInterface.pyd_field(
+            alias="i", piper_port=PiperPortType.INPUT
+        )
+    )
+
+    output: pl_interfaces.OutputDatasetInterface = (
+        pl_interfaces.OutputDatasetInterface.pyd_field(
+            alias="o", piper_port=PiperPortType.OUTPUT
+        )
+    )
+
+    algorithm: str = pyd.Field(
+        "sha256",
+        description=(
+            "The hashing algorithm from `hashlib` to use. Only algorithms that"
+            "do not require parameters are supported."
+        ),
+    )
+
+    keys: t.Union[str, t.Sequence[str]] = pyd.Field(
+        ...,
+        alias="k",
+        description=(
+            "The keys to use for comparison. All items selected must be equal to"
+            "consider two samples as duplicates."
+        ),
+    )
+
+    grabber: pl_interfaces.GrabberInterface = pl_interfaces.GrabberInterface.pyd_field(
+        alias="g"
+    )
+
+    def run(self):
+        from pipelime.stages import StageSampleHash
+        from pipelime.sequences import DataStream
+
+        seq = self.input.create_reader()
+        hash_key = self._get_hash_key(list(seq[0].keys()))
+        keys = [self.keys] if isinstance(self.keys, str) else self.keys
+        stage = StageSampleHash(algorithm=self.algorithm, keys=keys, hash_key=hash_key)
+        seq = seq.map(stage)
+
+        # multi-processing friendly filtering
+        class _WriterHelper:
+            def __init__(self, output_pipe):
+                self.stream = DataStream(output_pipe=output_pipe)
+                self.curr_idx = 0
+                self.unique_hashes = set()
+
+            def __call__(self, sample):
+                sample_hash = sample[hash_key]()
+                sample = sample.remove_keys(hash_key)
+                if sample_hash not in self.unique_hashes:
+                    self.unique_hashes.add(sample_hash)
+                    self.stream.set_output(self.curr_idx, sample)
+                    self.curr_idx += 1
+
+        if self.output.zfill is None:
+            self.output.zfill = seq.best_zfill()
+        writer_helper = _WriterHelper(output_pipe=self.output.as_pipe())
+
+        # filter out samples that have a hash that appears more than once
+        self.grabber.grab_all(
+            seq,
+            keep_order=True,
+            parent_cmd=self,
+            sample_fn=writer_helper,
+            track_message=f"Checking hashes ({len(seq)} samples)",
+        )
+
+    def _get_hash_key(self, keys: t.Sequence[str]) -> str:
+        """Get a key that has no conflict with the given keys.
+
+        Args:
+            keys (t.Sequence[str]): the keys of the samples
+
+        Returns:
+            str: the hash key
+        """
+        key = "hash"
+        while True:
+            if key not in keys:
+                return key
+            key += "_"
+
+
+class CopySharedItemsCommand(PipelimeCommand, title="copy-shared-items"):
+    """Copy shared items from a source dataset to a destination dataset. Datasets may
+    not have the same length."""
+
+    source: pl_interfaces.InputDatasetInterface = (
+        pl_interfaces.InputDatasetInterface.pyd_field(
+            alias="src",
+            piper_port=PiperPortType.INPUT,
+            description=(
+                "Where the shared items are copied from. Must have at least one "
+                "sample and one shared item."
+            ),
+        )
+    )
+    dest: pl_interfaces.InputDatasetInterface = (
+        pl_interfaces.InputDatasetInterface.pyd_field(
+            alias="dst",
+            piper_port=PiperPortType.INPUT,
+            description=(
+                "Where the shared items are copied to. Must have at least one sample "
+                "and any number of shared items. Source and destination datasets may "
+                "not have the same length, as only the shared items are copied."
+            ),
+        )
+    )
+    output: pl_interfaces.OutputDatasetInterface = (
+        pl_interfaces.OutputDatasetInterface.pyd_field(
+            alias="o",
+            piper_port=PiperPortType.OUTPUT,
+            description=("Where the resulting dataset is written to."),
+        )
+    )
+    grabber: pl_interfaces.GrabberInterface = pl_interfaces.GrabberInterface.pyd_field(
+        alias="g"
+    )
+    key_list: t.Sequence[str] = pyd.Field(
+        ...,
+        alias="k",
+        description=("The keys to copy. Must be present in source dataset."),
+    )
+    force_shared: bool = pyd.Field(
+        False,
+        alias="f",
+        description=("If True, the items will be copied as shared items"),
+    )
+
+    def run(self):
+        from pipelime.stages.item_replacement import StageCopyItems
+
+        src_seq = self.source.create_reader()
+        dst_seq = self.dest.create_reader()
+
+        src_sample = src_seq[0]
+
+        stage = StageCopyItems(
+            source=src_sample,
+            k=self.key_list,  # type: ignore
+            f=self.force_shared,  # type: ignore
+        )
+
+        out_seq = dst_seq.map(stage)
+
+        out_seq = self.output.append_writer(out_seq)
+
+        self.grabber.grab_all(
+            out_seq,
+            grab_context_manager=self.output.serialization_cm(),
+            keep_order=False,
+            parent_cmd=self,
+            track_message=f"Copying items ({len(out_seq)} samples)",
+        )
